@@ -55,7 +55,7 @@ using namespace std;
 
 #define UNSET_INT_VALUE INT_MIN
 
-	
+
 /**
  * Per-directory configuration information.
  *
@@ -65,23 +65,23 @@ using namespace std;
 struct DirConfig {
 	enum Threeway { ENABLED, DISABLED, UNSET };
 	enum SpawnMethod { SM_UNSET, SM_SMART, SM_DIRECT };
-	
+
 	#include "ConfigurationFields.hpp"
 
 	std::set<std::string> baseURIs;
-	
+
 	/** The path to the application's root (for example: RAILS_ROOT
 	 * for Rails applications, directory containing 'config.ru'
 	 * for Rack applications). If this value is NULL, the default
 	 * autodetected path will be used.
 	 */
 	const char *appRoot;
-	
+
 	string appGroupName;
-	
+
 	/** The spawn method to use. */
 	SpawnMethod spawnMethod;
-	
+
 	/**
 	 * The idle timeout, in seconds, of preloader processes.
 	 * May also be 0 (which indicates that the application spawner should
@@ -95,62 +95,62 @@ struct DirConfig {
 	 * "How Phusion Passenger detects whether a virtual host is a web application".
 	 */
 	Threeway resolveSymlinksInDocRoot;
-	
+
 	/**
 	 * Whether encoded slashes in URLs should be supported. This however conflicts
 	 * with mod_rewrite support because of a bug/limitation in Apache, so it's one
 	 * or the other.
 	 */
 	Threeway allowEncodedSlashes;
-	
+
 	/**
 	 * Throttle the number of stat() calls on files like
 	 * restart.txt to the once per given number of seconds.
 	 */
 	unsigned long statThrottleRate;
-	
+
 	/** Indicates whether the statThrottleRate option was
 	 * explicitly specified in the directory configuration. */
 	bool statThrottleRateSpecified;
-	
+
 	/** The directory in which Passenger should look for
 	 * restart.txt. NULL means that the default directory
 	 * should be used.
 	 */
 	const char *restartDir;
-	
+
 	/**
 	 * The directory in which Passenger should place upload buffer
 	 * files. NULL means that the default directory should be used.
 	 */
 	const char *uploadBufferDir;
-	
+
 	string unionStationKey;
-	
+
 	vector<string> unionStationFilters;
-	
+
 	/**
 	 * Whether Phusion Passenger should show friendly error pages.
 	 */
 	Threeway friendlyErrorPages;
-	
+
 	/**
 	 * Whether analytics logging should be enabled.
 	 */
 	Threeway unionStationSupport;
-	
+
 	/**
 	 * Whether response buffering support is enabled.
 	 */
 	Threeway bufferResponse;
-	
+
 	/*************************************/
 	/*************************************/
-	
+
 	bool isEnabled() const {
 		return enabled != DISABLED;
 	}
-	
+
 	StaticString getAppGroupName(const StaticString &appRoot) const {
 		if (appGroupName.empty()) {
 			return appRoot;
@@ -158,7 +158,7 @@ struct DirConfig {
 			return appGroupName;
 		}
 	}
-	
+
 	StaticString getSpawnMethodString() const {
 		switch (spawnMethod) {
 		case SM_SMART:
@@ -173,11 +173,11 @@ struct DirConfig {
 	bool highPerformanceMode() const {
 		return highPerformance == ENABLED;
 	}
-	
+
 	bool allowsEncodedSlashes() const {
 		return allowEncodedSlashes == ENABLED;
 	}
-	
+
 	unsigned long getStatThrottleRate() const {
 		if (statThrottleRateSpecified) {
 			return statThrottleRate;
@@ -185,7 +185,7 @@ struct DirConfig {
 			return 0;
 		}
 	}
-	
+
 	StaticString getRestartDir() const {
 		if (restartDir != NULL) {
 			return restartDir;
@@ -193,19 +193,21 @@ struct DirConfig {
 			return "";
 		}
 	}
-	
-	string getUploadBufferDir(const ServerInstanceDir::GenerationPtr &generation) const {
+
+	string getUploadBufferDir(const ServerInstanceDir::Generation *generation) const {
 		if (uploadBufferDir != NULL) {
 			return uploadBufferDir;
-		} else {
+		} else if (generation != NULL) {
 			return generation->getPath() + "/buffered_uploads";
+		} else {
+			return getSystemTempDir();
 		}
 	}
-	
+
 	bool showFriendlyErrorPages() const {
 		return friendlyErrorPages != DISABLED;
 	}
-	
+
 	bool useUnionStation() const {
 		return unionStationSupport == ENABLED;
 	}
@@ -213,14 +215,14 @@ struct DirConfig {
 	bool getBufferResponse() const {
 		return bufferResponse == ENABLED;
 	}
-	
+
 	string getUnionStationFilterString() const {
 		if (unionStationFilters.empty()) {
 			return string();
 		} else {
 			string result;
 			vector<string>::const_iterator it;
-			
+
 			for (it = unionStationFilters.begin(); it != unionStationFilters.end(); it++) {
 				if (it != unionStationFilters.begin()) {
 					result.append(1, '\1');
@@ -230,7 +232,7 @@ struct DirConfig {
 			return result;
 		}
 	}
-	
+
 	/*************************************/
 };
 
@@ -249,43 +251,43 @@ struct ServerConfig {
 
 	/** The default Ruby interpreter to use. */
 	const char *defaultRuby;
-	
+
 	/** The log verbosity. */
 	int logLevel;
-	
+
 	/** A file to print debug messages to, or NULL to just use STDERR. */
 	const char *debugLogFile;
-	
+
 	/** The maximum number of simultaneously alive application
 	 * instances. */
 	unsigned int maxPoolSize;
-	
+
 	/** The maximum number of seconds that an application may be
 	 * idle before it gets terminated. */
 	unsigned int poolIdleTime;
-	
+
 	/** Whether user switching support is enabled. */
 	bool userSwitching;
-	
+
 	/** See PoolOptions for more info. */
 	string defaultUser;
 	/** See PoolOptions for more info. */
 	string defaultGroup;
-	
+
 	/** The temp directory that Passenger should use. */
 	string tempDir;
-	
+
 	string unionStationGatewayAddress;
 	int unionStationGatewayPort;
 	string unionStationGatewayCert;
 	string unionStationProxyAddress;
-	
+
 	/** Directory in which analytics logs should be saved. */
 	string analyticsLogUser;
 	string analyticsLogGroup;
-	
+
 	set<string> prestartURLs;
-	
+
 	ServerConfig() {
 		root               = NULL;
 		defaultRuby        = DEFAULT_RUBY;
@@ -303,7 +305,7 @@ struct ServerConfig {
 		analyticsLogUser   = DEFAULT_ANALYTICS_LOG_USER;
 		analyticsLogGroup  = DEFAULT_ANALYTICS_LOG_GROUP;
 	}
-	
+
 	/** Called after the configuration files have been loaded, inside
 	 * the control process.
 	 */
@@ -315,7 +317,7 @@ struct ServerConfig {
 					string("The user that PassengerDefaultUser refers to, '") +
 					defaultUser + "', does not exist.");
 			}
-			
+
 			struct group *groupEntry = getgrgid(userEntry->pw_gid);
 			if (groupEntry == NULL) {
 				throw ConfigurationException(
@@ -324,7 +326,7 @@ struct ServerConfig {
 					"In other words, your system's user account database "
 					"is broken. Please fix it."));
 			}
-			
+
 			defaultGroup = groupEntry->gr_name;
 		}
 	}
