@@ -122,12 +122,12 @@ public:
 		 * failed it will transition to `DESTROYED`.
 		 */
 		INITIALIZING,
-		
+
 		/** This SuperGroup is loaded and is ready for action. From
 		 * here the state can transition to `RESTARTING` or `DESTROYING`.
 		 */
 		READY,
-		
+
 		/** This SuperGroup is being restarted. The SuperGroup
 		 * information is being reloaded from the data source
 		 * and processes are being restarted. In this state
@@ -143,7 +143,7 @@ public:
 		 * cause a transition to `DESTROYING`.
 		 */
 		RESTARTING,
-		
+
 		/** This SuperGroup is being destroyed. Processes are being shut
 		 * down and other resources are being cleaned up. In this state,
 		 * `groups` is empty.
@@ -151,7 +151,7 @@ public:
 		 * transition to `INITIALIZING`.
 		 */
 		DESTROYING,
-		
+
 		/** This SuperGroup has been destroyed and all resources have been
 		 * freed. Restarting won't have any effect but calling `get()` will
 		 * make it transition to `INITIALIZING`.
@@ -169,11 +169,11 @@ public:
 	};
 
 	typedef boost::function<void (ShutdownResult result)> ShutdownCallback;
-	
+
 private:
 	friend class Pool;
 	friend class Group;
-	
+
 	Options options;
 	/** A number for concurrency control, incremented every time the state changes.
 	 * Every background thread that SuperGroup spawns knows the generation number
@@ -186,8 +186,8 @@ private:
 	 * problems.
 	 */
 	unsigned int generation;
-	
-	
+
+
 	// Thread-safe.
 	static boost::mutex &getPoolSyncher(const PoolPtr &pool);
 	static void runAllActions(const vector<Callback> &actions);
@@ -195,13 +195,13 @@ private:
 	void runInitializationHooks() const;
 	void runDestructionHooks() const;
 	void setupInitializationOrDestructionHook(HookScriptOptions &options) const;
-	
+
 	void createInterruptableThread(const boost::function<void ()> &func, const string &name,
 		unsigned int stackSize);
-	
+
 	void verifyInvariants() const {
 		// !a || b: logical equivalent of a IMPLIES b.
-		
+
 		assert(groups.empty() ==
 			(state == INITIALIZING || state == DESTROYING || state == DESTROYED));
 		assert((defaultGroup == NULL) ==
@@ -210,7 +210,7 @@ private:
 			( getWaitlist.empty() ));
 		assert(!( state == DESTROYED ) || ( detachedGroups.empty() ));
 	}
-	
+
 	void setState(State newState) {
 		state = newState;
 		generation++;
@@ -224,10 +224,10 @@ private:
 		infos.push_back(info);
 		return infos;
 	}
-	
+
 	Group *findDefaultGroup(const vector<GroupPtr> &groups) const {
 		vector<GroupPtr>::const_iterator it;
-		
+
 		for (it = groups.begin(); it != groups.end(); it++) {
 			const GroupPtr &group = *it;
 			if (group->componentInfo.isDefault) {
@@ -236,7 +236,7 @@ private:
 		}
 		return NULL;
 	}
-	
+
 	pair<GroupPtr, unsigned int> findGroupCorrespondingToComponent(
 		const vector<GroupPtr> &groups, const ComponentInfo &info) const
 	{
@@ -249,7 +249,7 @@ private:
 		}
 		return make_pair(GroupPtr(), 0);
 	}
-	
+
 	static void oneGroupHasBeenShutDown(SuperGroupPtr self, GroupPtr group) {
 		// This function is either called from the pool event loop or directly from
 		// the detachAllGroups post lock actions. In both cases getPool() is never NULL.
@@ -264,7 +264,7 @@ private:
 			}
 		}
 	}
-	
+
 	/** One of the post lock actions can potentially perform a long-running
 	 * operation, so running them in a thread is advised.
 	 */
@@ -274,7 +274,7 @@ private:
 			if (group == NULL) {
 				continue;
 			}
-			
+
 			while (!group->getWaitlist.empty()) {
 				getWaitlist.push_back(group->getWaitlist.front());
 				group->getWaitlist.pop_front();
@@ -290,7 +290,7 @@ private:
 
 		groups.clear();
 	}
-	
+
 	void assignGetWaitlistToGroups(vector<Callback> &postLockActions) {
 		while (!getWaitlist.empty()) {
 			GetWaiter &waiter = getWaitlist.front();
@@ -306,7 +306,7 @@ private:
 			getWaitlist.pop_front();
 		}
 	}
-	
+
 	void adjustOptions(Options &options, const Group *group) const {
 		// No-op.
 	}
@@ -321,12 +321,12 @@ private:
 
 	void realDoInitialize(const Options &options, unsigned int generation);
 	void realDoRestart(const Options &options, unsigned int generation);
-	
+
 	void doDestroy(SuperGroupPtr self, unsigned int generation, ShutdownCallback callback) {
 		TRACE_POINT();
-		
+
 		runDestructionHooks();
-		
+
 		// Wait until 'detachedGroups' is empty.
 		UPDATE_TRACE_POINT();
 		PoolPtr pool = getPool();
@@ -350,7 +350,7 @@ private:
 				verifyInvariants();
 			}
 		}
-		
+
 		UPDATE_TRACE_POINT();
 		assert(state == DESTROYING);
 		state = DESTROYED;
@@ -361,29 +361,29 @@ private:
 			callback(SUCCESS);
 		}
 	}
-	
+
 	/*********************/
-	
+
 	/*********************/
-	
+
 public:
 	mutable boost::mutex backrefSyncher;
 	const boost::weak_ptr<Pool> pool;
-	
+
 	State state;
 	string name;
 	string secret;
-	
+
 	/** Invariant:
 	 * groups.empty() == (state == INITIALIZING || state == DESTROYING || state == DESTROYED)
 	 */
 	vector<GroupPtr> groups;
-	
+
 	/** Invariant:
 	 * (defaultGroup == NULL) == (state == INITIALIZING || state == DESTROYING || state == DESTROYED)
 	 */
 	Group *defaultGroup;
-	
+
 	/**
 	 * get() requests for this super group that cannot be immediately satisfied
 	 * are put on this wait list, which must be processed as soon as the
@@ -413,14 +413,14 @@ public:
 	 *       detachedGroups.empty()
 	 */
 	vector<GroupPtr> detachedGroups;
-	
+
 	/** One MUST call initialize() after construction because shared_from_this()
 	 * is not available in the constructor.
 	 */
 	SuperGroup(const PoolPtr &_pool, const Options &options)
 		: pool(_pool)
 	{
-		this->options = options.copyAndPersist().clearLogger();
+		this->options = options.copyAndPersist().detachFromUnionStationTransaction();
 		this->name = options.getAppGroupName();
 		secret = generateSecret();
 		state = INITIALIZING;
@@ -446,7 +446,7 @@ public:
 			"SuperGroup initializer: " + name,
 			POOL_HELPER_THREAD_STACK_SIZE);
 	}
-	
+
 	/**
 	 * Thread-safe.
 	 *
@@ -462,7 +462,7 @@ public:
 	bool isAlive() const {
 		return state != DESTROYING && state != DESTROYED;
 	}
-	
+
 	const char *getStateName() const {
 		switch (state) {
 		case INITIALIZING:
@@ -551,7 +551,7 @@ public:
 			verifyInvariants();
 		}
 	}
-	
+
 	/**
 	 * @post
 	 *    if result:
@@ -561,7 +561,7 @@ public:
 		/* if (state == READY) {
 			vector<GroupPtr>::const_iterator it, end = groups.end();
 			bool result = true;
-			
+
 			for (it = groups.begin(); result && it != end; it++) {
 				result = result && (*it)->garbageCollectable(now);
 			}
@@ -573,7 +573,7 @@ public:
 		} */
 		return false;
 	}
-	
+
 	SessionPtr get(const Options &newOptions, const GetCallback &callback,
 		vector<Callback> &postLockActions)
 	{
@@ -605,7 +605,7 @@ public:
 				boost::bind(
 					doInitialize,
 					shared_from_this(),
-					newOptions.copyAndPersist().clearLogger(),
+					newOptions.copyAndPersist().detachFromUnionStationTransaction(),
 					generation),
 				"SuperGroup initializer: " + name,
 				POOL_HELPER_THREAD_STACK_SIZE);
@@ -616,15 +616,15 @@ public:
 			return SessionPtr(); // Shut up compiler warning.
 		};
 	}
-	
+
 	Group *route(const Options &options) const {
 		return defaultGroup;
 	}
-	
+
 	unsigned int capacityUsed() const {
 		vector<GroupPtr>::const_iterator it, end = groups.end();
 		unsigned int result = 0;
-		
+
 		for (it = groups.begin(); it != end; it++) {
 			result += (*it)->capacityUsed();
 		}
@@ -647,7 +647,7 @@ public:
 	bool needsRestart() const {
 		return false;
 	}
-	
+
 	void restart(const Options &options) {
 		verifyInvariants();
 		if (state == READY) {
@@ -656,7 +656,7 @@ public:
 					doRestart,
 					// Keep reference to self to prevent destruction.
 					shared_from_this(),
-					options.copyAndPersist().clearLogger(),
+					options.copyAndPersist().detachFromUnionStationTransaction(),
 					generation),
 				"SuperGroup restarter: " + name,
 				POOL_HELPER_THREAD_STACK_SIZE);
