@@ -62,6 +62,7 @@
 #include <Utils.h>
 #include <Utils/ScopeGuard.h>
 #include <Utils/IOUtils.h>
+#include <Utils/StringScanning.h>
 
 namespace Passenger {
 
@@ -73,7 +74,7 @@ using namespace oxt;
 struct ProcessMetrics {
 	pid_t   pid;
 	pid_t   ppid;
-	uint8_t cpu;
+	boost::uint8_t cpu;
 	/** Resident Set Size, amount of memory in RAM. Does not include swap.
 	 * -1 if not yet known, 0 if completely swapped out.
 	 */
@@ -91,11 +92,11 @@ struct ProcessMetrics {
 	 */
 	ssize_t  swap;
 	/** OS X Snow Leopard does not report the VM size correctly, so don't use this. */
-	size_t  vmsize;
+	ssize_t  vmsize;
 	pid_t   processGroupId;
 	uid_t   uid;
 	string  command;
-	
+
 	ProcessMetrics() {
 		pid = (pid_t) -1;
 		rss = -1;
@@ -104,11 +105,11 @@ struct ProcessMetrics {
 		swap = -1;
 		vmsize = -1;
 	}
-	
+
 	bool isValid() const {
 		return pid != (pid_t) -1;
 	}
-	
+
 	/**
 	 * Returns an estimate of the "real" memory usage of a process in KB.
 	 * We don't use the PSS here because that would mean if another
@@ -147,33 +148,33 @@ public:
 	size_t totalMemory(ssize_t &shared) const {
 		const_iterator it, end = this->end();
 		bool pssAndPrivateDirtyAvailable = true;
-		
+
 		for (it = begin(); it != end && pssAndPrivateDirtyAvailable; it++) {
 			const ProcessMetrics &metric = it->second;
 			pssAndPrivateDirtyAvailable = pssAndPrivateDirtyAvailable &&
 				metric.pss != -1 && metric.privateDirty != -1;
 		}
-		
+
 		if (pssAndPrivateDirtyAvailable) {
 			size_t total = 0;
 			size_t priv = 0;
-			
+
 			for (it = begin(); it != end; it++) {
 				const ProcessMetrics &metric = it->second;
 				total += metric.pss;
 				priv += metric.privateDirty;
 			}
-			
+
 			shared = total - priv;
 			return total;
 		} else {
 			size_t total = 0;
-			
+
 			for (it = begin(); it != end; it++) {
 				const ProcessMetrics &metric = it->second;
 				total += metric.realMemory();
 			}
-			
+
 			shared = -1;
 			return total;
 		}
@@ -185,183 +186,18 @@ public:
  * command name, etc.
  */
 class ProcessMetricsCollector {
-public:
-	struct ParseException {};
-	
 private:
 	bool canMeasureRealMemory;
 	string psOutput;
-	
-	/**
-	 * Scan the given data for the first word that appears on the first line.
-	 * Leading whitespaces (but not newlines) are ignored. If a word is found
-	 * then the word is returned and the data pointer is moved to the end of
-	 * the word. Otherwise, a ParseException is thrown.
-	 *
-	 * @post result.size() > 0
-	 */
-	static StaticString readNextWord(const char **data) {
-		// Skip leading whitespaces.
-		while (**data == ' ') {
-			(*data)++;
-		}
-		if (**data == '\n' || **data == '\0') {
-			throw ParseException();
-		}
-		
-		// Find end of word and extract the word.
-		const char *endOfWord = *data;
-		while (*endOfWord != ' ' && *endOfWord != '\n' && *endOfWord != '\0') {
-			endOfWord++;
-		}
-		StaticString result(*data, endOfWord - *data);
-		
-		// Move data pointer to the end of this word.
-		*data = endOfWord;
-		return result;
-	}
 
-	static long long processNextWordAsLongLong(const StaticString &word, char *nullTerminatedWord) {
-		memcpy(nullTerminatedWord, word.c_str(), word.size());
-		nullTerminatedWord[word.size()] = '\0';
-		if (*nullTerminatedWord == '\0') {
-			throw ParseException();
-		} else {
-			return atoll(nullTerminatedWord);
-		}
-	}
-	
-	static long long readNextWordAsLongLong(const char **data) {
-		StaticString word = readNextWord(data);
-		if (word.size() < 50) {
-			char nullTerminatedWord[50];
-			return processNextWordAsLongLong(word, nullTerminatedWord);
-		} else {
-			string nullTerminatedWord(word.size() + 1, '\0');
-			return processNextWordAsLongLong(word, &nullTerminatedWord[0]);
-		}
-	}
-
-	static int processNextWordAsInt(const StaticString &word, char *nullTerminatedWord) {
-		memcpy(nullTerminatedWord, word.c_str(), word.size());
-		nullTerminatedWord[word.size()] = '\0';
-		if (*nullTerminatedWord == '\0') {
-			throw ParseException();
-		} else {
-			return atoi(nullTerminatedWord);
-		}
-	}
-	
-	static int readNextWordAsInt(const char **data) {
-		StaticString word = readNextWord(data);
-		if (word.size() < 50) {
-			char nullTerminatedWord[50];
-			return processNextWordAsInt(word, nullTerminatedWord);
-		} else {
-			string nullTerminatedWord(word.size() + 1, '\0');
-			return processNextWordAsInt(word, &nullTerminatedWord[0]);
-		}
-	}
-	
-	string runCommandAndCaptureOutput(const char **command) const {
-		pid_t pid;
-		int e;
-		Pipe p;
-		
-		p = createPipe();
-		
-		this_thread::disable_syscall_interruption dsi;
-		pid = syscalls::fork();
-		if (pid == 0) {
-			// Make ps nicer, we want to have as little impact on the rest
-			// of the system as possible while collecting the metrics.
-			int prio = getpriority(PRIO_PROCESS, getpid());
-			prio++;
-			if (prio > 20) {
-				prio = 20;
-			}
-			setpriority(PRIO_PROCESS, getpid(), prio);
-			
-			dup2(p[1], 1);
-			close(p[0]);
-			close(p[1]);
-			closeAllFileDescriptors(2);
-			execvp(command[0], (char * const *) command);
-			_exit(1);
-		} else if (pid == -1) {
-			e = errno;
-			throw SystemException("Cannot fork() a new process", e);
-		} else {
-			bool done = false;
-			string result;
-			
-			p[1].close();
-			while (!done) {
-				char buf[1024 * 4];
-				ssize_t ret;
-				
-				try {
-					this_thread::restore_syscall_interruption rsi(dsi);
-					ret = syscalls::read(p[0], buf, sizeof(buf));
-				} catch (const thread_interrupted &) {
-					syscalls::kill(SIGKILL, pid);
-					syscalls::waitpid(pid, NULL, 0);
-					throw;
-				}
-				if (ret == -1) {
-					e = errno;
-					syscalls::kill(SIGKILL, pid);
-					syscalls::waitpid(pid, NULL, 0);
-					throw SystemException("Cannot read output from the 'ps' command", e);
-				}
-				done = ret == 0;
-				result.append(buf, ret);
-			}
-			p[0].close();
-			syscalls::waitpid(pid, NULL, 0);
-			
-			if (result.empty()) {
-				throw RuntimeException("The 'ps' command failed");
-			} else {
-				return result;
-			}
-		}
-	}
-	
-	string readRestOfLine(const char *data) const {
-		// Skip leading whitespaces.
-		while (*data == ' ') {
-			data++;
-		}
-		// Rest of line is allowed to be empty.
-		if (*data == '\n' || *data == '\0') {
-			return "";
-		}
-		
-		// Look for newline character. From there, scan back until we've
-		// found a non-whitespace character.
-		const char *endOfLine = strchr(data, '\n');
-		if (endOfLine == NULL) {
-			throw ParseException();
-		}
-		while (*(endOfLine - 1) == ' ') {
-			endOfLine--;
-		}
-		
-		return string(data, endOfLine - data);
-	}
-	
 	template<typename Collection, typename ConstIterator>
 	ProcessMetricMap parsePsOutput(const string &output, const Collection &allowedPids) const {
 		ProcessMetricMap result;
+		const char *start = output.c_str();
+
 		// Ignore first line, it contains the column names.
-		const char *start = strchr(output.c_str(), '\n');
-		if (start != NULL) {
-			// Skip to beginning of next line.
-			start++;
-			if (*start == '\0') {
-				start = NULL;
-			}
+		if (!skipToNextLine(&start) || *start == '\0') {
+			start = NULL;
 		}
 
 		#ifndef PS_SUPPORTS_MULTIPLE_PIDS
@@ -371,11 +207,11 @@ private:
 				pids.insert(*it);
 			}
 		#endif
-		
+
 		// Parse each line.
 		while (start != NULL) {
 			ProcessMetrics metrics;
-			
+
 			metrics.pid  = (pid_t) readNextWordAsLongLong(&start);
 			metrics.ppid = (pid_t) readNextWordAsLongLong(&start);
 			metrics.cpu  = readNextWordAsInt(&start);
@@ -394,7 +230,7 @@ private:
 
 			if (pidAllowed) {
 				result[metrics.pid] = metrics;
-				
+
 				start = strchr(start, '\n');
 				if (start != NULL) {
 					// Skip to beginning of next line.
@@ -407,7 +243,7 @@ private:
 		}
 		return result;
 	}
-	
+
 public:
 	ProcessMetricsCollector() {
 		#ifdef __APPLE__
@@ -416,19 +252,19 @@ public:
 			canMeasureRealMemory = fileExists("/proc/self/smaps");
 		#endif
 	}
-	
+
 	/** Mock 'ps' output, used by unit tests. */
 	void setPsOutput(const string &data) {
 		this->psOutput = data;
 	}
-	
+
 	/**
 	 * Collect metrics for the given process IDs. Nonexistant PIDs are not
 	 * included in the result.
 	 *
 	 * Returns a map which maps a given PID to its collected metrics.
 	 *
-	 * @throws ProcessMetricsCollector::ParseException
+	 * @throws ParseException The ps output cannot be parsed.
 	 * @throws SystemException
 	 * @throws RuntimeException
 	 */
@@ -437,12 +273,12 @@ public:
 		if (pids.empty()) {
 			return ProcessMetricMap();
 		}
-		
+
 		ConstIterator it;
 		// The list of PIDs must follow -p without a space.
 		// https://groups.google.com/forum/#!topic/phusion-passenger/WKXy61nJBMA
 		string pidsArg = "-p";
-		
+
 		for (it = pids.begin(); it != pids.end(); it++) {
 			pidsArg.append(toString(*it));
 			pidsArg.append(",");
@@ -450,25 +286,31 @@ public:
 		if (pidsArg[pidsArg.size() - 1] == ',') {
 			pidsArg.resize(pidsArg.size() - 1);
 		}
-		
+
+		// The list of format arguments must also follow -o
+		// without a space.
+		// https://github.com/phusion/passenger/pull/94
+		string fmtArg = "-o";
+		#if defined(sun) || defined(__sun)
+			fmtArg.append("pid,ppid,pcpu,rss,vsz,pgid,uid,args");
+		#else
+			fmtArg.append("pid,ppid,%cpu,rss,vsize,pgid,uid,command");
+		#endif
+
 		const char *command[] = {
-			"ps", "-o",
-			#if defined(sun) || defined(__sun)
-				"pid,ppid,pcpu,rss,vsz,pgid,uid,args",
-			#else
-				"pid,ppid,%cpu,rss,vsize,pgid,uid,command",
-			#endif
+			"ps", fmtArg.c_str(),
 			#ifdef PS_SUPPORTS_MULTIPLE_PIDS
 				pidsArg.c_str(),
 			#endif
 			NULL
 		};
-		
+
 		string psOutput = this->psOutput;
 		if (psOutput.empty()) {
 			psOutput = runCommandAndCaptureOutput(command);
 		}
 		pidsArg.resize(0);
+		fmtArg.resize(0);
 		ProcessMetricMap result = parsePsOutput<Collection, ConstIterator>(psOutput, pids);
 		psOutput.resize(0);
 		if (canMeasureRealMemory) {
@@ -481,11 +323,11 @@ public:
 		}
 		return result;
 	}
-	
+
 	ProcessMetricMap collect(const vector<pid_t> &pids) const {
 		return collect< vector<pid_t>, vector<pid_t>::const_iterator >(pids);
 	}
-	
+
 	/**
 	 * Attempt to measure various parts of a process's memory usage that may
 	 * contribute to insight as to what its "real" memory usage might be.
@@ -495,7 +337,7 @@ public:
 	 *   sharing it.
 	 * - The private dirty RSS.
 	 * - Amount of memory in swap.
-	 * 
+	 *
 	 * At this time only OS X and recent Linux versions (>= 2.6.25) support
 	 * measuring the proportional set size. Usually root privileges are required.
 	 *
@@ -507,35 +349,35 @@ public:
 		#ifdef __APPLE__
 			kern_return_t ret;
 			mach_port_t task;
-			
+
 			swap = -1;
-			
+
 			ret = task_for_pid(mach_task_self(), pid, &task);
 			if (ret != KERN_SUCCESS) {
 				pss = -1;
 				privateDirty = -1;
 				return;
 			}
-			
+
 			mach_vm_address_t addr = 0;
 			int pagesize = getpagesize();
-			
+
 			// In bytes.
 			pss = 0;
 			privateDirty = 0;
-			
+
 			while (true) {
 				mach_vm_address_t size;
 				vm_region_top_info_data_t info;
 				mach_msg_type_number_t count = VM_REGION_TOP_INFO_COUNT;
 				mach_port_t object_name;
-				
+
 				ret = mach_vm_region(task, &addr, &size, VM_REGION_TOP_INFO,
 					(vm_region_info_t) &info, &count, &object_name);
 				if (ret != KERN_SUCCESS) {
 					break;
 				}
-				
+
 				if (info.share_mode == SM_PRIVATE) {
 					// shared_pages_resident here means that region
 					// has shared memory only "shared" between 1 process.
@@ -549,12 +391,12 @@ public:
 				} else if (info.share_mode == SM_SHARED) {
 					pss += info.shared_pages_resident * pagesize / info.ref_count;
 				}
-				
+
 				addr += size;
 			}
-			
+
 			mach_port_deallocate(mach_task_self(), task);
-			
+
 			// Convert result back to KB.
 			pss /= 1024;
 			privateDirty /= 1024;
@@ -562,7 +404,7 @@ public:
 			string smapsFilename = "/proc/";
 			smapsFilename.append(toString(pid));
 			smapsFilename.append("/smaps");
-			
+
 			FILE *f = syscalls::fopen(smapsFilename.c_str(), "r");
 			if (f == NULL) {
 				error:
@@ -571,21 +413,21 @@ public:
 				swap = -1;
 				return;
 			}
-			
+
 			StdioGuard guard(f);
 			bool hasPss = false;
 			bool hasPrivateDirty = false;
 			bool hasSwap = false;
-			
+
 			// In KB.
 			pss = 0;
 			privateDirty = 0;
 			swap = 0;
-			
+
 			while (!feof(f)) {
 				char line[1024 * 4];
 				const char *buf;
-				
+
 				buf = fgets(line, sizeof(line), f);
 				if (buf == NULL) {
 					if (ferror(f)) {
@@ -624,7 +466,7 @@ public:
 					goto error;
 				}
 			}
-			
+
 			if (!hasPss) {
 				pss = -1;
 			}
