@@ -121,14 +121,6 @@ static void *customDiagnosticsDumperUserData;
 static int emergencyPipe1[2] = { -1, -1 };
 static int emergencyPipe2[2] = { -1, -1 };
 
-// If assert() failed, its information is stored here.
-static struct {
-	const char *filename;
-	const char *function; // May be NULL.
-	const char *expression;
-	unsigned int line;
-} lastAssertionFailure;
-
 
 static void
 ignoreSigpipe() {
@@ -217,7 +209,7 @@ appendULL(char *buf, unsigned long long value) {
 		remainder = remainder / 10;
 		size++;
 	} while (remainder != 0);
-	
+
 	reverse(buf, size);
 	return buf + size;
 }
@@ -228,13 +220,13 @@ static char *
 appendIntegerAsHex(char *buf, IntegerType value) {
 	IntegerType remainder = value;
 	unsigned int size = 0;
-	
+
 	do {
 		buf[size] = hex_chars[remainder % 16];
 		remainder = remainder / 16;
 		size++;
 	} while (remainder != 0);
-	
+
 	reverse(buf, size);
 	return buf + size;
 }
@@ -298,7 +290,7 @@ appendSignalName(char *buf, int signo) {
 static char *
 appendSignalReason(char *buf, siginfo_t *info) {
 	bool handled = true;
-	
+
 	switch (info->si_code) {
 	SI_CODE_HANDLER(SI_USER);
 	#ifdef SI_KERNEL
@@ -359,7 +351,7 @@ appendSignalReason(char *buf, siginfo_t *info) {
 		}
 		break;
 	}
-	
+
 	if (info->si_code <= 0) {
 		buf = appendText(buf, ", signal sent by PID ");
 		buf = appendULL(buf, (unsigned long long) info->si_pid);
@@ -369,7 +361,7 @@ appendSignalReason(char *buf, siginfo_t *info) {
 
 	buf = appendText(buf, ", si_addr=");
 	buf = appendPointerAsString(buf, info->si_addr);
-	
+
 	return buf;
 }
 
@@ -438,7 +430,7 @@ dumpFileDescriptorInfoWithLsof(AbortHandlerState &state, void *userData) {
 	end = appendULL(end, state.pid);
 	*end = '\0';
 
-	closeAllFileDescriptors(2);
+	closeAllFileDescriptors(2, true);
 
 	execlp("lsof", "lsof", "-p", state.messageBuf, "-nP", (const char * const) 0);
 
@@ -457,7 +449,7 @@ dumpFileDescriptorInfoWithLs(AbortHandlerState &state, char *end) {
 
 	pid = asyncFork();
 	if (pid == 0) {
-		closeAllFileDescriptors(2);
+		closeAllFileDescriptors(2, true);
 		// The '-v' is for natural sorting on Linux. On BSD -v means something else but it's harmless.
 		execlp("ls", "ls", "-lv", state.messageBuf, (const char * const) 0);
 		_exit(1);
@@ -514,10 +506,10 @@ dumpWithCrashWatch(AbortHandlerState &state) {
 	char *end = messageBuf;
 	end = appendULL(end, (unsigned long long) state.pid);
 	*end = '\0';
-	
+
 	pid_t child = asyncFork();
 	if (child == 0) {
-		closeAllFileDescriptors(2);
+		closeAllFileDescriptors(2, true);
 		execlp("crash-watch", "crash-watch", "--dump", pidStr, (char * const) 0);
 		if (errno == ENOENT) {
 			safePrintErr("Crash-watch is not installed. Please install it with 'gem install crash-watch' "
@@ -583,8 +575,8 @@ dumpWithCrashWatch(AbortHandlerState &state) {
 
 				close(p[1]);
 				dup2(p[0], STDIN_FILENO);
-				closeAllFileDescriptors(2);
-				
+				closeAllFileDescriptors(2, true);
+
 				char *command = end;
 				end = appendText(end, "exec ");
 				end = appendText(end, backtraceSanitizerCommand);
@@ -664,7 +656,7 @@ dumpDiagnostics(AbortHandlerState &state) {
 	// Dump human-readable time string and string.
 	pid = asyncFork();
 	if (pid == 0) {
-		closeAllFileDescriptors(2);
+		closeAllFileDescriptors(2, true);
 		execlp("date", "date", (const char * const) 0);
 		_exit(1);
 	} else if (pid == -1) {
@@ -676,7 +668,7 @@ dumpDiagnostics(AbortHandlerState &state) {
 	// Dump system uname.
 	pid = asyncFork();
 	if (pid == 0) {
-		closeAllFileDescriptors(2);
+		closeAllFileDescriptors(2, true);
 		execlp("uname", "uname", "-mprsv", (const char * const) 0);
 		_exit(1);
 	} else if (pid == -1) {
@@ -688,7 +680,7 @@ dumpDiagnostics(AbortHandlerState &state) {
 	// Dump ulimit.
 	pid = asyncFork();
 	if (pid == 0) {
-		closeAllFileDescriptors(2);
+		closeAllFileDescriptors(2, true);
 		execlp("ulimit", "ulimit", "-a", (const char * const) 0);
 		// On Linux 'ulimit' is a shell builtin, not a command.
 		execlp("/bin/sh", "/bin/sh", "-c", "ulimit -a", (const char * const) 0);
@@ -929,7 +921,7 @@ abortHandler(int signo, siginfo_t *info, void *ctx) {
 
 		child = asyncFork();
 		if (child == 0) {
-			closeAllFileDescriptors(2);
+			closeAllFileDescriptors(2, true);
 			#ifdef __APPLE__
 				execlp("osascript", "osascript", "-e", "beep 2", (const char * const) 0);
 				safePrintErr("Cannot execute 'osascript' command\n");
@@ -1046,7 +1038,7 @@ abortHandler(int signo, siginfo_t *info, void *ctx) {
 	 * We work around this for anything in the same linkage unit by just definin
 	 * our own versions of the assert handler and abort.
 	 */
-	
+
 	#include <pthread.h>
 
 	extern "C" int
@@ -1093,7 +1085,7 @@ installAbortHandler() {
 		fflush(stderr);
 		abort();
 	}
-	
+
 	stack_t stack;
 	stack.ss_sp = alternativeStack;
 	stack.ss_size = alternativeStackSize;
@@ -1105,7 +1097,7 @@ installAbortHandler() {
 		fflush(stderr);
 		abort();
 	}
-	
+
 	struct sigaction action;
 	action.sa_sigaction = abortHandler;
 	action.sa_flags = SA_RESETHAND | SA_SIGINFO;
@@ -1406,7 +1398,7 @@ initializeSyscallFailureSimulation(const char *processName) {
 	string prefix = string(processName) + "=";
 	vector<string> components;
 	unsigned int i;
-	
+
 	// Lookup this process in the specification string.
 	split(spec, ';', components);
 	for (i = 0; i < components.size(); i++) {
@@ -1495,7 +1487,7 @@ initializeAgent(int argc, char *argv[], const char *processName) {
 	}
 	srand(randomSeed);
 	srandom(randomSeed);
-	
+
 	ignoreSigpipe();
 	if (hasEnvOption("PASSENGER_ABORT_HANDLER", true)) {
 		shouldDumpWithCrashWatch = hasEnvOption("PASSENGER_DUMP_WITH_CRASH_WATCH", true);
@@ -1512,7 +1504,7 @@ initializeAgent(int argc, char *argv[], const char *processName) {
 	}
 	setvbuf(stdout, NULL, _IONBF, 0);
 	setvbuf(stderr, NULL, _IONBF, 0);
-	
+
 	TRACE_POINT();
 	try {
 		if (argc == 1) {
@@ -1544,7 +1536,7 @@ initializeAgent(int argc, char *argv[], const char *processName) {
 		} else {
 			options.readFrom((const char **) argv + 1, argc - 1);
 		}
-		
+
 		#ifdef __linux__
 			if (options.has("passenger_root")) {
 				ResourceLocator locator(options.get("passenger_root", true));
@@ -1569,14 +1561,14 @@ initializeAgent(int argc, char *argv[], const char *processName) {
 				 */
 				string filename = options.get("debug_log_file");
 				options.erase("debug_log_file");
-				
+
 				int fd = open(filename.c_str(), O_CREAT | O_WRONLY | O_APPEND, 0644);
 				if (fd == -1) {
 					int e = errno;
 					throw FileSystemException("Cannot open debug log file " +
 						filename, e, filename);
 				}
-				
+
 				dup2(fd, STDOUT_FILENO);
 				dup2(fd, STDERR_FILENO);
 				close(fd);
@@ -1588,7 +1580,7 @@ initializeAgent(int argc, char *argv[], const char *processName) {
 		P_ERROR("*** ERROR: " << e.what() << "\n" << e.backtrace());
 		exit(1);
 	}
-	
+
 	// Change process title.
 	argv0 = strdup(argv[0]);
 	strncpy(argv[0], processName, strlen(argv[0]));
@@ -1597,7 +1589,7 @@ initializeAgent(int argc, char *argv[], const char *processName) {
 	}
 
 	P_DEBUG("Random seed: " << randomSeed);
-	
+
 	return options;
 }
 
