@@ -1,6 +1,6 @@
 /*
  *  Phusion Passenger - https://www.phusionpassenger.com/
- *  Copyright (c) 2010-2013 Phusion
+ *  Copyright (c) 2010-2014 Phusion
  *
  *  "Phusion Passenger" is a trademark of Hongli Lai & Ninh Bui.
  *
@@ -72,14 +72,14 @@ class LoggingServer: public EventedMessageServer {
 private:
 	static const int MAX_LOG_SINK_CACHE_SIZE = 512;
 	static const int GARBAGE_COLLECTION_TIMEOUT = 4500;  // 1 hour 15 minutes
-	
+
 	struct LogSink;
 	typedef boost::shared_ptr<LogSink> LogSinkPtr;
 	typedef map<string, LogSinkPtr> LogSinkCache;
-	
+
 	struct LogSink {
 		LoggingServer *server;
-		
+
 		/**
 		 * Marks how many times this LogSink is currently opened, i.e. the
 		 * number of Transaction objects currently referencing this LogSink.
@@ -87,27 +87,27 @@ private:
 		 *    (opened == 0) == (this LogSink is in LoggingServer.inactiveLogSinks)
 		 */
 		int opened;
-		
+
 		/** Last time this LogSink hit an open count of 0. */
 		ev_tstamp lastUsed;
-		
+
 		/** Last time data was actually written to the underlying storage device. */
 		ev_tstamp lastFlushed;
 
 		/** The amount of data that has been written to this sink so far. */
 		unsigned int writtenTo;
-		
+
 		/**
 		 * This LogSink's iterator inside LoggingServer.logSinkCache.
 		 */
 		LogSinkCache::iterator cacheIterator;
-		
+
 		/**
 		 * This LogSink's iterator inside LoggingServer.inactiveLogSinks.
 		 * Only valid when opened == 0.
 		 */
 		list<LogSinkPtr>::iterator inactiveLogSinksIterator;
-		
+
 		LogSink(LoggingServer *_server) {
 			server = _server;
 			opened = 0;
@@ -115,39 +115,39 @@ private:
 			lastFlushed = lastUsed;
 			writtenTo = 0;
 		}
-		
+
 		virtual ~LogSink() {
 			// We really want to flush() here but can't call virtual
 			// functions in destructor. :(
 		}
-		
+
 		virtual bool isRemote() const {
 			return false;
 		}
 
 		// Default interval at which this sink should be flushed.
 		virtual unsigned int defaultFlushInterval() const {
-			return 15;
+			return 5;
 		}
-		
+
 		virtual void append(const DataStoreId &dataStoreId,
 			const StaticString &data)
 		{
 			writtenTo += data.size();
 		}
-		
+
 		virtual bool flush() {
 			lastFlushed = ev_now(server->getLoop());
 			return true;
 		}
-		
+
 		virtual void dump(ostream &stream) const { }
 	};
-	
+
 	struct LogFileSink: public LogSink {
 		string filename;
 		FileDescriptor fd;
-		
+
 		LogFileSink(LoggingServer *server, const string &filename)
 			: LogSink(server)
 		{
@@ -164,11 +164,11 @@ private:
 				throw FileSystemException("Cannnot open file", e, filename);
 			}
 		}
-		
+
 		virtual ~LogFileSink() {
 			flush();
 		}
-		
+
 		virtual void append(const DataStoreId &dataStoreId, const StaticString &data) {
 			LogSink::append(dataStoreId, data);
 			syscalls::write(fd, data.data(), data.size());
@@ -182,9 +182,9 @@ private:
 			stream << "     WrittenTo  : " << writtenTo << "\n";
 		}
 	};
-	
+
 	typedef boost::shared_ptr<LogFileSink> LogFileSinkPtr;
-	
+
 	struct RemoteSink: public LogSink {
 		/* RemoteSender compresses the data with zlib before sending it
 		 * to the server. Even including Base64 and URL encoding overhead,
@@ -202,13 +202,13 @@ private:
 		static const unsigned int BUFFER_CAPACITY =
 			4 * 64 * 1024 -
 			16 * 1024;
-		
+
 		string unionStationKey;
 		string nodeName;
 		string category;
 		char buffer[BUFFER_CAPACITY];
 		unsigned int bufferSize;
-		
+
 		RemoteSink(LoggingServer *server, const string &unionStationKey,
 			const string &nodeName, const string &category)
 			: LogSink(server)
@@ -218,26 +218,26 @@ private:
 			this->category = category;
 			this->bufferSize = 0;
 		}
-		
+
 		virtual ~RemoteSink() {
 			flush();
 		}
-		
+
 		virtual bool isRemote() const {
 			return true;
 		}
 
 		virtual unsigned int defaultFlushInterval() const {
-			return 60;
+			return 5;
 		}
-		
+
 		virtual void append(const DataStoreId &dataStoreId, const StaticString &data) {
 			LogSink::append(dataStoreId, data);
 			if (bufferSize + data.size() > BUFFER_CAPACITY) {
 				StaticString data2[2];
 				data2[0] = StaticString(buffer, bufferSize);
 				data2[1] = data;
-				
+
 				server->remoteSender.schedule(unionStationKey, nodeName,
 					category, data2, 2);
 				lastFlushed = ev_now(server->getLoop());
@@ -247,7 +247,7 @@ private:
 				bufferSize += data.size();
 			}
 		}
-		
+
 		virtual bool flush() {
 			if (bufferSize > 0) {
 				lastFlushed = ev_now(server->getLoop());
@@ -266,7 +266,7 @@ private:
 		string inspect() const {
 			return "(key=" + unionStationKey + ", node=" + nodeName + ", category=" + category + ")";
 		}
-		
+
 		virtual void dump(ostream &stream) const {
 			stream << "   * Remote sink\n";
 			stream << "     Key        : " << unionStationKey << "\n";
@@ -279,7 +279,7 @@ private:
 			stream << "     BufferSize : " << bufferSize << "\n";
 		}
 	};
-	
+
 	struct Transaction {
 		LoggingServer *server;
 		LogSinkPtr logSink;
@@ -291,13 +291,13 @@ private:
 		bool crashProtect, discarded;
 		string data;
 		string filters;
-		
+
 		Transaction(LoggingServer *server, ev_tstamp createdAt) {
 			this->server = server;
 			this->createdAt = createdAt;
 			data.reserve(8 * 1024);
 		}
-		
+
 		~Transaction() {
 			if (logSink != NULL) {
 				if (!discarded && passesFilter()) {
@@ -306,24 +306,24 @@ private:
 				server->closeLogSink(logSink);
 			}
 		}
-		
+
 		StaticString getGroupName() const {
 			return dataStoreId.getGroupName();
 		}
-		
+
 		StaticString getNodeName() const {
 			return dataStoreId.getNodeName();
 		}
-		
+
 		StaticString getCategory() const {
 			return dataStoreId.getCategory();
 		}
-		
+
 		void discard() {
 			data.clear();
 			discarded = true;
 		}
-		
+
 		void dump(ostream &stream) const {
 			stream << "   * Transaction " << txnId << "\n";
 			stream << "     Created at: " << distanceOfTimeInWords((time_t) createdAt) << " ago\n";
@@ -332,18 +332,18 @@ private:
 			stream << "     Category  : " << getCategory() << "\n";
 			stream << "     Refcount  : " << refcount << "\n";
 		}
-	
+
 	private:
 		bool passesFilter() {
 			if (filters.empty()) {
 				return true;
 			}
-			
+
 			const char *current = filters.data();
 			const char *end     = filters.data() + filters.size();
 			bool result         = true;
 			FilterSupport::ContextFromLog ctx(data);
-			
+
 			// 'filters' may contain multiple filter sources, separated
 			// by '\1' characters. Process each.
 			while (current < end && result) {
@@ -352,24 +352,24 @@ private:
 				if (pos == string::npos) {
 					pos = tmp.size();
 				}
-				
+
 				StaticString source(current, pos);
 				FilterSupport::Filter &filter = server->compileFilter(source);
 				result = filter.run(ctx);
-				
+
 				current = tmp.data() + pos + 1;
 			}
 			return result;
 		}
 	};
-	
+
 	typedef boost::shared_ptr<Transaction> TransactionPtr;
-	
+
 	enum ClientType {
 		UNINITIALIZED,
 		LOGGER
 	};
-	
+
 	struct Client: public EventedMessageClient {
 		string nodeName;
 		ClientType type;
@@ -382,7 +382,7 @@ private:
 		ScalarMessage dataReader;
 		TransactionPtr currentTransaction;
 		string currentTimestamp;
-		
+
 		Client(struct ev_loop *loop, const FileDescriptor &fd)
 			: EventedMessageClient(loop, fd)
 		{
@@ -406,12 +406,12 @@ private:
 			stream << "     Outbox           : " << outbox.size() << " bytes\n";
 		}
 	};
-	
+
 	typedef boost::shared_ptr<Client> ClientPtr;
 	typedef map<string, TransactionPtr> TransactionMap;
-	
+
 	typedef boost::shared_ptr<FilterSupport::Filter> FilterPtr;
-	
+
 	RemoteSender remoteSender;
 	ev::timer garbageCollectionTimer;
 	ev::timer sinkFlushingTimer;
@@ -434,12 +434,12 @@ private:
 	unsigned long long exitBeginTime;
 	int sinkFlushInterval;
 	string dumpFile;
-	
+
 	void sendErrorToClient(Client *client, const string &message) {
 		client->writeArrayMessage("error", message.c_str(), NULL);
 		logError(client, message);
 	}
-	
+
 	bool expectingArgumentsCount(Client *client, const vector<StaticString> &args, unsigned int size) {
 		if (args.size() == size) {
 			return true;
@@ -449,7 +449,7 @@ private:
 			return false;
 		}
 	}
-	
+
 	bool expectingMinArgumentsCount(Client *client, const vector<StaticString> &args, unsigned int size) {
 		if (args.size() >= size) {
 			return true;
@@ -459,7 +459,7 @@ private:
 			return false;
 		}
 	}
-	
+
 	bool expectingLoggerType(Client *client) {
 		if (client->type == LOGGER) {
 			return true;
@@ -469,7 +469,7 @@ private:
 			return false;
 		}
 	}
-	
+
 	bool checkWhetherConnectionAreAcceptable(Client *client) {
 		if (refuseNewConnections) {
 			client->writeArrayMessage("server shutting down", NULL);
@@ -479,7 +479,7 @@ private:
 			return true;
 		}
 	}
-	
+
 	static bool getBool(const vector<StaticString> &args, unsigned int index,
 		bool defaultValue = false)
 	{
@@ -489,7 +489,7 @@ private:
 			return defaultValue;
 		}
 	}
-	
+
 	static StaticString getStaticString(const vector<StaticString> &args,
 		unsigned int index, const StaticString &defaultValue = "")
 	{
@@ -499,7 +499,7 @@ private:
 			return defaultValue;
 		}
 	}
-	
+
 	bool validTxnId(const StaticString &txnId) const {
 		// must contain timestamp
 		// must contain separator
@@ -507,13 +507,13 @@ private:
 		// must not be too large
 		return !txnId.empty();
 	}
-	
+
 	bool validUnionStationKey(const StaticString &key) const {
 		// TODO: must be hexadecimal
 		// TODO: must not be too large
 		return !key.empty();
 	}
-	
+
 	bool validLogContent(const StaticString &data) const {
 		const char *current = data.c_str();
 		const char *end = current + data.size();
@@ -526,17 +526,18 @@ private:
 		}
 		return true;
 	}
-	
+
 	bool validTimestamp(const StaticString &timestamp) const {
 		// TODO: must be hexadecimal
 		// TODO: must not be too large
 		return true;
 	}
-	
+
 	bool supportedCategory(const StaticString &category) const {
-		return category == "requests" || category == "processes" || category == "exceptions";
+		return category == "requests" || category == "processes"
+			|| category == "exceptions" || category == "system_metrics";
 	}
-	
+
 	LogSinkPtr openLogFile() {
 		string cacheKey = "file:" + dumpFile;
 		LogSinkPtr result;
@@ -558,7 +559,7 @@ private:
 		}
 		return result;
 	}
-	
+
 	LogSinkPtr openRemoteSink(const StaticString &unionStationKey, const string &nodeName,
 		const string &category)
 	{
@@ -568,7 +569,7 @@ private:
 		cacheKey.append(nodeName);
 		cacheKey.append(1, '\0');
 		cacheKey.append(category);
-		
+
 		LogSinkPtr result;
 		LogSinkCache::iterator it = logSinkCache.find(cacheKey);
 		if (it == logSinkCache.end()) {
@@ -589,7 +590,7 @@ private:
 		}
 		return result;
 	}
-	
+
 	/**
 	 * 'Closes' the given log sink. It's not actually deleted from memory;
 	 * instead it's marked as inactive and cached for later use. May be
@@ -610,7 +611,7 @@ private:
 			trimLogSinkCache(MAX_LOG_SINK_CACHE_SIZE);
 		}
 	}
-	
+
 	/** Try to reduce the log sink cache size to the given size. */
 	void trimLogSinkCache(unsigned int size) {
 		while (!inactiveLogSinks.empty() && logSinkCache.size() > size) {
@@ -620,7 +621,7 @@ private:
 			logSinkCache.erase(logSink->cacheIterator);
 		}
 	}
-	
+
 	FilterSupport::Filter &compileFilter(const StaticString &source) {
 		// TODO: garbage collect filters based on time
 		FilterPtr filter = filters.get(source);
@@ -630,7 +631,7 @@ private:
 		}
 		return *filter;
 	}
-	
+
 	bool writeLogEntry(Client *client, const TransactionPtr &transaction,
 		const StaticString &timestamp, const StaticString &data)
 	{
@@ -651,7 +652,7 @@ private:
 			}
 			return false;
 		}
-		
+
 		char writeCountStr[sizeof(unsigned int) * 2 + 1];
 		integerToHexatri(transaction->writeCount, writeCountStr);
 		transaction->writeCount++;
@@ -665,7 +666,7 @@ private:
 		transaction->data.append("\n");
 		return true;
 	}
-	
+
 	void writeDetachEntry(Client *client, const TransactionPtr &transaction) {
 		char timestamp[2 * sizeof(unsigned long long) + 1];
 		// Must use System::getUsec() here instead of ev_now() because the
@@ -673,13 +674,13 @@ private:
 		integerToHexatri<unsigned long long>(SystemTime::getUsec(), timestamp);
 		writeDetachEntry(client, transaction, timestamp);
 	}
-	
+
 	void writeDetachEntry(Client *client, const TransactionPtr &transaction,
 		const StaticString &timestamp)
 	{
 		writeLogEntry(client, transaction, timestamp, "DETACH");
 	}
-	
+
 	bool requireRights(Client *client, Account::Rights rights) {
 		if (client->messageServer.account->hasRights(rights)) {
 			return true;
@@ -692,7 +693,7 @@ private:
 			return false;
 		}
 	}
-	
+
 	bool isDirectory(const string &dir, struct dirent *entry) const {
 		#if defined(__sun__) || defined(sun) || defined(_AIX)
 			string path = dir;
@@ -703,7 +704,7 @@ private:
 			return entry->d_type == DT_DIR;
 		#endif
 	}
-	
+
 	bool looksLikeNumber(const char *str) const {
 		const char *current = str;
 		while (*current != '\0') {
@@ -715,13 +716,13 @@ private:
 		}
 		return true;
 	}
-	
+
 	/* Release all inactive log sinks that have been inactive for more than
 	 * GARBAGE_COLLECTION_TIMEOUT seconds.
 	 */
 	void releaseInactiveLogSinks(ev_tstamp now) {
 		bool done = false;
-		
+
 		while (!done && !inactiveLogSinks.empty()) {
 			const LogSinkPtr logSink = inactiveLogSinks.front();
 			if (now - logSink->lastUsed >= GARBAGE_COLLECTION_TIMEOUT) {
@@ -733,7 +734,7 @@ private:
 			}
 		}
 	}
-	
+
 	void garbageCollect(ev::timer &timer, int revents) {
 		P_DEBUG("Garbage collection time");
 		releaseInactiveLogSinks(ev_now(getLoop()));
@@ -746,13 +747,13 @@ private:
 			return sinkFlushInterval;
 		}
 	}
-	
+
 	void sinkFlushTimeout(ev::timer &timer, int revents) {
 		P_DEBUG("Flushing all sinks");
 		LogSinkCache::iterator it;
 		LogSinkCache::iterator end = logSinkCache.end();
 		ev_tstamp now = ev_now(getLoop());
-		
+
 		for (it = logSinkCache.begin(); it != end; it++) {
 			LogSink *sink = it->second.get();
 			if (now - sink->lastFlushed >= getFlushInterval(sink)) {
@@ -760,18 +761,18 @@ private:
 			}
 		}
 	}
-	
+
 	void flushAllSinks() {
 		P_TRACE(2, "Flushing all sinks");
 		LogSinkCache::iterator it;
 		LogSinkCache::iterator end = logSinkCache.end();
-		
+
 		for (it = logSinkCache.begin(); it != end; it++) {
 			LogSink *sink = it->second.get();
 			sink->flush();
 		}
 	}
-	
+
 	void exitTimerTimeout(ev::timer &timer, int revents) {
 		if (SystemTime::getMsec() >= exitBeginTime + 5000) {
 			exitTimer.stop();
@@ -780,24 +781,24 @@ private:
 			ev_break(getLoop(), EVBREAK_ONE);
 		}
 	}
-	
+
 protected:
 	virtual EventedClient *createClient(const FileDescriptor &fd) {
 		return new Client(getLoop(), fd);
 	}
-	
+
 	virtual bool onMessageReceived(EventedMessageClient *_client, const vector<StaticString> &args) {
 		Client *client = (Client *) _client;
-		
+
 		if (args[0] == "log") {
 			if (OXT_UNLIKELY( !expectingArgumentsCount(client, args, 3)
 			               || !expectingLoggerType(client) )) {
 				return true;
 			}
-			
+
 			string txnId     = args[1];
 			string timestamp = args[2];
-			
+
 			TransactionMap::iterator it = transactions.find(txnId);
 			if (OXT_UNLIKELY( it == transactions.end() )) {
 				sendErrorToClient(client, "Cannot log data: transaction does not exist");
@@ -815,13 +816,13 @@ protected:
 				client->currentTimestamp = timestamp;
 				return false;
 			}
-			
+
 		} else if (args[0] == "openTransaction") {
 			if (OXT_UNLIKELY( !expectingMinArgumentsCount(client, args, 7)
 			               || !expectingLoggerType(client) )) {
 				return true;
 			}
-			
+
 			string       txnId     = args[1];
 			StaticString groupName = args[2];
 			StaticString nodeName  = args[3];
@@ -831,7 +832,7 @@ protected:
 			bool         crashProtect    = getBool(args, 7, true);
 			bool         ack             = getBool(args, 8, false);
 			StaticString filters         = getStaticString(args, 9);
-			
+
 			if (OXT_UNLIKELY( !validTxnId(txnId) )) {
 				sendErrorToClient(client, "Invalid transaction ID format");
 				client->disconnect();
@@ -850,16 +851,16 @@ protected:
 				client->disconnect();
 				return true;
 			}
-			
+
 			const char *nodeId;
-			
+
 			if (nodeName.empty()) {
 				nodeName = client->nodeName;
 				nodeId = client->nodeId;
 			} else {
 				nodeId = NULL;
 			}
-			
+
 			TransactionMap::iterator it = transactions.find(txnId);
 			TransactionPtr transaction;
 			if (it == transactions.end()) {
@@ -868,11 +869,11 @@ protected:
 					client->disconnect();
 					return true;
 				}
-				
+
 				transaction = boost::make_shared<Transaction>(this, ev_now(getLoop()));
 				if (unionStationKey.empty() || unionStationKey == "-") {
 					char tempNodeId[MD5_HEX_SIZE];
-					
+
 					if (nodeId == NULL) {
 						md5_state_t state;
 						md5_byte_t  digest[MD5_SIZE];
@@ -886,7 +887,7 @@ protected:
 							tempNodeId);
 						nodeId = tempNodeId;
 					}
-					
+
 					transaction->logSink = openLogFile();
 				} else {
 					transaction->logSink = openRemoteSink(unionStationKey,
@@ -926,25 +927,25 @@ protected:
 					return true;
 				}
 			}
-			
+
 			client->openTransactions.insert(txnId);
 			transaction->refcount++;
 			writeLogEntry(client, transaction, timestamp, "ATTACH");
-			
+
 			if (ack) {
 				client->writeArrayMessage("ok", NULL);
 			}
-			
+
 		} else if (args[0] == "closeTransaction") {
 			if (OXT_UNLIKELY( !expectingMinArgumentsCount(client, args, 3)
 			               || !expectingLoggerType(client) )) {
 				return true;
 			}
-			
+
 			string txnId = args[1];
 			StaticString timestamp = args[2];
 			bool         ack       = getBool(args, 3, false);
-			
+
 			TransactionMap::iterator it = transactions.find(txnId);
 			if (OXT_UNLIKELY( it == transactions.end() )) {
 				sendErrorToClient(client,
@@ -953,7 +954,7 @@ protected:
 				client->disconnect();
 			} else {
 				TransactionPtr &transaction = it->second;
-				
+
 				set<string>::const_iterator sit = client->openTransactions.find(txnId);
 				if (OXT_UNLIKELY( sit == client->openTransactions.end() )) {
 					sendErrorToClient(client,
@@ -964,7 +965,7 @@ protected:
 				} else {
 					client->openTransactions.erase(sit);
 				}
-				
+
 				writeDetachEntry(client, transaction, timestamp);
 				transaction->refcount--;
 				assert(transaction->refcount >= 0);
@@ -972,11 +973,11 @@ protected:
 					transactions.erase(it);
 				}
 			}
-			
+
 			if (ack) {
 				client->writeArrayMessage("ok", NULL);
 			}
-		
+
 		} else if (args[0] == "init") {
 			if (OXT_UNLIKELY( client->type != UNINITIALIZED )) {
 				sendErrorToClient(client, "Already initialized");
@@ -989,32 +990,32 @@ protected:
 			if (OXT_UNLIKELY( !checkWhetherConnectionAreAcceptable(client) )) {
 				return true;
 			}
-			
+
 			StaticString nodeName = args[1];
 			client->nodeName = nodeName;
-			
+
 			md5_state_t state;
 			md5_byte_t  digest[MD5_SIZE];
 			md5_init(&state);
 			md5_append(&state, (const md5_byte_t *) nodeName.data(), nodeName.size());
 			md5_finish(&state, digest);
 			toHex(StaticString((const char *) digest, MD5_SIZE), client->nodeId);
-			
+
 			client->type = LOGGER;
 			client->writeArrayMessage("ok", NULL);
-			
+
 		} else if (args[0] == "flush") {
 			flushAllSinks();
 			client->writeArrayMessage("ok", NULL);
-			
+
 		} else if (args[0] == "info") {
 			stringstream stream;
 			dump(stream);
 			client->writeArrayMessage("info", stream.str().c_str(), NULL);
-		
+
 		} else if (args[0] == "ping") {
 			client->writeArrayMessage("pong", NULL);
-		
+
 		} else if (args[0] == "exit") {
 			if (!requireRights(client, Account::EXIT)) {
 				client->disconnect();
@@ -1036,15 +1037,15 @@ protected:
 				exitRequested = true;
 			}
 			client->disconnect();
-			
+
 		} else {
 			sendErrorToClient(client, "Unknown command '" + args[0] + "'");
 			client->disconnect();
 		}
-		
+
 		return true;
 	}
-	
+
 	virtual pair<size_t, bool> onOtherDataReceived(EventedMessageClient *_client,
 		const char *data, size_t size)
 	{
@@ -1064,20 +1065,20 @@ protected:
 			return make_pair(consumed, false);
 		}
 	}
-	
+
 	virtual void onNewClient(EventedClient *client) {
 		if (exitRequested && exitTimer.is_active()) {
 			exitTimer.stop();
 		}
 		EventedMessageServer::onNewClient(client);
 	}
-	
+
 	virtual void onClientDisconnected(EventedClient *_client) {
 		EventedMessageServer::onClientDisconnected(_client);
 		Client *client = (Client *) _client;
 		set<string>::const_iterator sit;
 		set<string>::const_iterator send = client->openTransactions.end();
-		
+
 		// Close any transactions that this client had opened.
 		for (sit = client->openTransactions.begin(); sit != send; sit++) {
 			const string &txnId = *sit;
@@ -1086,7 +1087,7 @@ protected:
 				P_ERROR("Bug: client->openTransactions is not a subset of this->transactions!");
 				abort();
 			}
-			
+
 			TransactionPtr &transaction = it->second;
 			if (transaction->crashProtect) {
 				writeDetachEntry(client, transaction);
@@ -1100,7 +1101,7 @@ protected:
 			}
 		}
 		client->openTransactions.clear();
-		
+
 		// Possibly start exit timer.
 		if (exitRequested && getClients().empty()) {
 			exitTimer.start();
@@ -1128,7 +1129,7 @@ public:
 		  exitTimer(loop),
 		  dumpFile(options.get("analytics_dump_file", false, "/dev/null"))
 	{
-		int sinkFlushTimerInterval = options.getInt("analytics_sink_flush_timer_interval", false, 15);
+		int sinkFlushTimerInterval = options.getInt("analytics_sink_flush_timer_interval", false, 5);
 		sinkFlushInterval = options.getInt("analytics_sink_flush_interval", false, 0);
 		garbageCollectionTimer.set<LoggingServer, &LoggingServer::garbageCollect>(this);
 		garbageCollectionTimer.start(GARBAGE_COLLECTION_TIMEOUT, GARBAGE_COLLECTION_TIMEOUT);
@@ -1140,7 +1141,7 @@ public:
 		exitRequested = false;
 		inactiveLogSinksCount = 0;
 	}
-	
+
 	~LoggingServer() {
 		TransactionMap::iterator it, end = transactions.end();
 		for (it = transactions.begin(); it != end; it++) {
@@ -1151,18 +1152,18 @@ public:
 				transaction->discard();
 			}
 		}
-		
+
 		// Invoke destructors, causing all transactions and log sinks to
 		// be flushed before RemoteSender is being destroyed.
 		transactions.clear();
 		logSinkCache.clear();
 		inactiveLogSinks.clear();
 	}
-	
+
 	void dump(ostream &stream) const {
 		TransactionMap::const_iterator it;
 		TransactionMap::const_iterator end = transactions.end();
-		
+
 		ClientSet::const_iterator cit, cend = getClients().end();
 		stream << "Clients:\n";
 		stream << "  Count: " << getClients().size() << "\n";
