@@ -1,6 +1,6 @@
 /*
  *  Phusion Passenger - https://www.phusionpassenger.com/
- *  Copyright (c) 2010-2013 Phusion
+ *  Copyright (c) 2010-2014 Phusion
  *
  *  "Phusion Passenger" is a trademark of Hongli Lai & Ninh Bui.
  *
@@ -45,6 +45,7 @@
 #include <Utils/SystemTime.h>
 #include <Utils/ScopeGuard.h>
 #include <Utils/Base64.h>
+#include <Utils/json.h>
 #include <Utils/Curl.h>
 
 namespace Passenger {
@@ -63,29 +64,29 @@ private:
 		string nodeName;
 		string category;
 		string data;
-		
+
 		Item() {
 			exit = false;
 			compressed = false;
 		}
 	};
-	
+
 	class Server {
 	private:
 		string ip;
 		unsigned short port;
 		string certificate;
 		const CurlProxyInfo *proxyInfo;
-		
+
 		CURL *curl;
 		struct curl_slist *headers;
 		char lastErrorMessage[CURL_ERROR_SIZE];
 		string hostHeader;
 		string responseBody;
-		
+
 		string pingURL;
 		string sinkURL;
-		
+
 		void resetConnection() {
 			if (curl != NULL) {
 				#ifdef HAS_CURL_EASY_RESET
@@ -121,18 +122,72 @@ private:
 			setCurlProxy(curl, *proxyInfo);
 			responseBody.clear();
 		}
-		
+
 		void prepareRequest(const string &url) {
 			curl_easy_setopt(curl, CURLOPT_URL, url.c_str());
 			responseBody.clear();
 		}
-		
+
+		static bool validateResponse(const Json::Value &response) {
+			if (response.isObject() && response["status"].isString()) {
+				string status = response["status"].asString();
+				if (status == "ok") {
+					return true;
+				} else if (status == "error") {
+					return response["message"].isString();
+				} else {
+					return false;
+				}
+			} else {
+				return false;
+			}
+		}
+
+		bool handleSendResponse() {
+			Json::Reader reader;
+			Json::Value response;
+			long httpCode = -1;
+
+			curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &httpCode);
+
+			if (!reader.parse(responseBody, response, false) || !validateResponse(response)) {
+				P_ERROR("The Union Station gateway server " << ip << " encountered an error "
+					"while processing sent analytics data. It sent an invalid response "
+					"(parse error: " << reader.getFormattedErrorMessages().c_str() <<
+					"; HTTP code: " << httpCode <<
+					"; data: \"" << cEscapeString(responseBody) << "\").");
+				return false;
+			} else if (response["status"].asString() == "ok") {
+				if (httpCode == 200) {
+					P_DEBUG("The Union Station gateway server " << ip << " accepted the packet.");
+					return true;
+				} else {
+					P_ERROR("The Union Station gateway server " << ip << " encountered an error "
+						"while processing sent analytics data. It sent an invalid response "
+						"(HTTP code: " << httpCode <<
+						"; data: \"" << cEscapeString(responseBody) << "\").");
+					return false;
+				}
+			} else {
+				// response == error
+				P_ERROR("The Union Station gateway server " << ip << " did not accept the "
+					"sent analytics data. Error message: " << response["message"].asString());
+				// Return value of true is intentional. See comment for send().
+				return true;
+			}
+		}
+
+		void handleSendError() {
+			P_ERROR("Could not send data to Union Station gateway server " << ip
+				<< ": " << lastErrorMessage);
+		}
+
 		static size_t curlDataReceived(void *buffer, size_t size, size_t nmemb, void *userData) {
 			Server *self = (Server *) userData;
 			self->responseBody.append((const char *) buffer, size * nmemb);
 			return size * nmemb;
 		}
-		
+
 	public:
 		Server(const string &ip, const string &hostName, unsigned short port, const string &cert,
 			const CurlProxyInfo *proxyInfo)
@@ -141,25 +196,25 @@ private:
 			this->port = port;
 			certificate = cert;
 			this->proxyInfo = proxyInfo;
-			
+
 			hostHeader = "Host: " + hostName;
 			headers = NULL;
 			headers = curl_slist_append(headers, hostHeader.c_str());
 			if (headers == NULL) {
 				throw IOException("Unable to create a CURL linked list");
 			}
-			
-			// Older libcurl versions didn't strdup() any option 
+
+			// Older libcurl versions didn't strdup() any option
 			// strings so we need to keep these in memory.
 			pingURL = string("https://") + ip + ":" + toString(port) +
 				"/ping";
 			sinkURL = string("https://") + ip + ":" + toString(port) +
 				"/sink";
-			
+
 			curl = NULL;
 			resetConnection();
 		}
-		
+
 		~Server() {
 			if (curl != NULL) {
 				curl_easy_cleanup(curl);
@@ -170,12 +225,12 @@ private:
 		string name() const {
 			return ip + ":" + toString(port);
 		}
-		
+
 		bool ping() {
 			P_DEBUG("Pinging Union Station gateway " << ip << ":" << port);
 			ScopeGuard guard(boost::bind(&Server::resetConnection, this));
 			prepareRequest(pingURL);
-			
+
 			curl_easy_setopt(curl, CURLOPT_HTTPGET, 1);
 			if (curl_easy_perform(curl) != 0) {
 				P_DEBUG("Could not ping Union Station gateway server " << ip
@@ -192,15 +247,23 @@ private:
 				return false;
 			}
 		}
-		
+
+		/** Returns true if the server is up, false if the server is down.
+		 * The return value does NOT indicate whether the server accepted the data!
+		 * Thus, if (for example) the Union Station key is invalid or disabled,
+		 * but the connection is fine, then this method still returns true.
+		 * This is because the return value is used to determine whether a different
+		 * gateway server should be used. If the server is up but rejects the data
+		 * then we'll want the code to keep sending future packets.
+		 */
 		bool send(const Item &item) {
 			ScopeGuard guard(boost::bind(&Server::resetConnection, this));
 			prepareRequest(sinkURL);
-			
+
 			struct curl_httppost *post = NULL;
 			struct curl_httppost *last = NULL;
 			string base64_data;
-			
+
 			curl_formadd(&post, &last,
 				CURLFORM_PTRNAME, "key",
 				CURLFORM_PTRCONTENTS, item.unionStationKey.c_str(),
@@ -234,7 +297,7 @@ private:
 					CURLFORM_CONTENTSLENGTH, (long) item.data.size(),
 					CURLFORM_END);
 			}
-			
+
 			curl_easy_setopt(curl, CURLOPT_HTTPGET, 0);
 			curl_easy_setopt(curl, CURLOPT_HTTPPOST, post);
 			P_DEBUG("Sending Union Station packet: key=" << item.unionStationKey <<
@@ -242,47 +305,45 @@ private:
 				", compressedDataSize=" << item.data.size());
 			CURLcode code = curl_easy_perform(curl);
 			curl_formfree(post);
-			
+
 			if (code == CURLE_OK) {
 				guard.clear();
-				// TODO: check response
-				return true;
+				return handleSendResponse();
 			} else {
-				P_DEBUG("Could not send data to Union Station gateway server " << ip
-					<< ": " << lastErrorMessage);
+				handleSendError();
 				return false;
 			}
 		}
 	};
-	
+
 	typedef boost::shared_ptr<Server> ServerPtr;
-	
+
 	string gatewayAddress;
 	unsigned short gatewayPort;
 	string certificate;
 	CurlProxyInfo proxyInfo;
 	BlockingQueue<Item> queue;
 	oxt::thread *thr;
-	
+
 	mutable boost::mutex syncher;
 	list<ServerPtr> servers;
 	time_t nextCheckupTime;
 	unsigned int packetsSent, packetsDropped;
-	
+
 	void threadMain() {
 		ScopeGuard guard(boost::bind(&RemoteSender::freeThreadData, this));
-		
+
 		while (true) {
 			Item item;
 			bool hasItem;
-			
+
 			if (firstStarted()) {
 				item = queue.get();
 				hasItem = true;
 			} else {
 				hasItem = queue.timedGet(item, msecUntilNextCheckup());
 			}
-			
+
 			if (hasItem) {
 				if (item.exit) {
 					return;
@@ -297,24 +358,24 @@ private:
 			}
 		}
 	}
-	
+
 	bool firstStarted() const {
 		boost::lock_guard<boost::mutex> l(syncher);
 		return nextCheckupTime == 0;
 	}
-	
+
 	void recheckServers() {
 		P_INFO("Rechecking Union Station gateway servers (" << gatewayAddress << ")...");
-		
+
 		vector<string> ips;
 		vector<string>::const_iterator it;
 		list<ServerPtr> servers;
 		string hostName;
 		bool someServersAreDown = false;
-		
+
 		ips = resolveHostname(gatewayAddress, gatewayPort);
 		P_INFO(ips.size() << " Union Station gateway servers found");
-		
+
 		for (it = ips.begin(); it != ips.end(); it++) {
 			ServerPtr server = boost::make_shared<Server>(*it, gatewayAddress, gatewayPort,
 				certificate, &proxyInfo);
@@ -325,7 +386,7 @@ private:
 			}
 		}
 		P_INFO(servers.size() << " Union Station gateway servers are up");
-		
+
 		if (servers.empty()) {
 			scheduleNextCheckup(5 * 60);
 		} else if (someServersAreDown) {
@@ -337,12 +398,12 @@ private:
 		boost::lock_guard<boost::mutex> l(syncher);
 		this->servers = servers;
 	}
-	
+
 	void freeThreadData() {
 		boost::lock_guard<boost::mutex> l(syncher);
 		servers.clear(); // Invoke destructors inside this thread.
 	}
-	
+
 	/**
 	 * Schedules the next checkup to be run after the given number
 	 * of seconds, unless there's already a checkup scheduled for
@@ -355,7 +416,7 @@ private:
 			P_DEBUG("Next checkup time in about " << seconds << " seconds");
 		}
 	}
-	
+
 	unsigned int msecUntilNextCheckup() const {
 		boost::lock_guard<boost::mutex> l(syncher);
 		time_t now = SystemTime::get();
@@ -365,17 +426,17 @@ private:
 			return (nextCheckupTime - now) * 1000;
 		}
 	}
-	
+
 	bool timeForCheckup() const {
 		boost::lock_guard<boost::mutex> l(syncher);
 		return SystemTime::get() >= nextCheckupTime;
 	}
-	
+
 	void sendOut(const Item &item) {
 		boost::unique_lock<boost::mutex> l(syncher);
 		bool sent = false;
 		bool someServersWentDown = false;
-		
+
 		while (!sent && !servers.empty()) {
 			// Pick first available server and put it on the back of the list
 			// for round-robin load balancing.
@@ -394,7 +455,7 @@ private:
 				packetsDropped++;
 			}
 		}
-		
+
 		if (someServersWentDown) {
 			if (servers.empty()) {
 				scheduleNextCheckup(5 * 60);
@@ -402,7 +463,7 @@ private:
 				scheduleNextCheckup(60 * 60);
 			}
 		}
-		
+
 		/* If all servers went down then all items in the queue will be
 		 * effectively dropped until after the next checkup has detected
 		 * servers that are up.
@@ -415,18 +476,18 @@ private:
 				", compressedDataSize=" << item.data.size());
 		}
 	}
-	
+
 	bool compress(const StaticString data[], unsigned int count, string &output) {
 		if (count == 0) {
 			StaticString newdata;
 			return compress(&newdata, 1, output);
 		}
-		
+
 		unsigned char out[128 * 1024];
 		z_stream strm;
 		int ret, flush;
 		unsigned int i, have;
-		
+
 		strm.zalloc = Z_NULL;
 		strm.zfree  = Z_NULL;
 		strm.opaque = Z_NULL;
@@ -434,12 +495,12 @@ private:
 		if (ret != Z_OK) {
 			return false;
 		}
-		
+
 		for (i = 0; i < count; i++) {
 			strm.avail_in = data[i].size();
 			strm.next_in  = (unsigned char *) data[i].c_str();
 			flush = (i == count - 1) ? Z_FINISH : Z_NO_FLUSH;
-			
+
 			do {
 				strm.avail_out = sizeof(out);
 				strm.next_out  = out;
@@ -451,11 +512,11 @@ private:
 			assert(strm.avail_in == 0);
 		}
 		assert(ret == Z_STREAM_END);
-		
+
 		deflateEnd(&strm);
 		return true;
 	}
-	
+
 public:
 	RemoteSender(const string &gatewayAddress, unsigned short gatewayPort, const string &certificate,
 		const string &proxyAddress)
@@ -480,7 +541,7 @@ public:
 			1024 * 512
 		);
 	}
-	
+
 	~RemoteSender() {
 		Item item;
 		item.exit = true;
@@ -493,7 +554,7 @@ public:
 		thr->join();
 		delete thr;
 	}
-	
+
 	void schedule(const string &unionStationKey, const StaticString &nodeName,
 		const StaticString &category, const StaticString data[],
 		unsigned int count)
@@ -503,13 +564,13 @@ public:
 		item.unionStationKey = unionStationKey;
 		item.nodeName = nodeName;
 		item.category = category;
-		
+
 		if (compress(data, count, item.data)) {
 			item.compressed = true;
 		} else {
 			size_t size = 0;
 			unsigned int i;
-			
+
 			for (i = 0; i < count; i++) {
 				size += data[i].size();
 			}
@@ -518,7 +579,7 @@ public:
 				item.data.append(data[i].c_str(), data[i].size());
 			}
 		}
-		
+
 		P_DEBUG("Scheduling Union Station packet: key=" << unionStationKey <<
 			", node=" << nodeName << ", category=" << category <<
 			", compressedDataSize=" << item.data.size());
@@ -529,7 +590,7 @@ public:
 			packetsDropped++;
 		}
 	}
-	
+
 	unsigned int queued() const {
 		return queue.size();
 	}
