@@ -1,5 +1,5 @@
 #  Phusion Passenger - https://www.phusionpassenger.com/
-#  Copyright (c) 2010-2013 Phusion
+#  Copyright (c) 2010-2014 Phusion
 #
 #  "Phusion Passenger" is a trademark of Hongli Lai & Ninh Bui.
 #
@@ -20,15 +20,18 @@
 #  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 #  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 #  THE SOFTWARE.
+PhusionPassenger.require_passenger_lib 'standalone/utils'
 PhusionPassenger.require_passenger_lib 'utils/file_system_watcher'
 
 module PhusionPassenger
 module Standalone
 
 class AppFinder
+	include Standalone::Utils
+
 	attr_accessor :dirs
 	attr_reader :apps
-	
+
 	def self.looks_like_app_directory?(dir)
 		return File.exist?("#{dir}/config.ru") ||
 			File.exist?("#{dir}/config/environment.rb") ||
@@ -36,7 +39,7 @@ class AppFinder
 			File.exist?("#{dir}/app.js") ||
 			File.exist?("#{dir}/.meteor")
 	end
-	
+
 	def initialize(dirs, options = {})
 		@dirs = dirs
 		@options = options.dup
@@ -45,11 +48,11 @@ class AppFinder
 	def global_options
 		return @options
 	end
-	
+
 	def scan
 		apps = []
 		watchlist = []
-		
+
 		if single_mode?
 			app_root = find_app_root
 			apps << {
@@ -59,7 +62,7 @@ class AppFinder
 			watchlist << app_root
 			watchlist << "#{app_root}/config" if File.exist?("#{app_root}/config")
 			watchlist << "#{app_root}/passenger-standalone.json" if File.exist?("#{app_root}/passenger-standalone.json")
-			
+
 			config_filename = File.join(app_root, "passenger-standalone.json")
 			if File.exist?(config_filename)
 				global_options = load_config_file!(:global_config, config_filename)
@@ -75,15 +78,15 @@ class AppFinder
 		@watchlist = watchlist
 		return apps
 	end
-	
+
 	def monitor(termination_pipe)
 		raise "You must call #scan first" if !@apps
-		
+
 		watcher = PhusionPassenger::Utils::FileSystemWatcher.new(@watchlist, termination_pipe)
 		if wait_on_io(termination_pipe, 3)
 			return
 		end
-		
+
 		while true
 			changed = watcher.wait_for_change
 			watcher.close
@@ -94,13 +97,13 @@ class AppFinder
 				if wait_on_io(termination_pipe, 0.25)
 					return
 				end
-				
+
 				new_apps = scan
 				watcher = PhusionPassenger::Utils::FileSystemWatcher.new(@watchlist, termination_pipe)
 				if old_apps != new_apps
 					yield(new_apps)
 				end
-				
+
 				# Don't process change events again for a short while,
 				# but do detect changes while waiting.
 				if wait_on_io(termination_pipe, 3)
@@ -117,11 +120,11 @@ class AppFinder
 	def single_mode?
 		return true
 	end
-	
+
 	def multi_mode?
 		return !single_mode?
 	end
-	
+
 	##################
 
 private
@@ -130,12 +133,12 @@ private
 
 	def find_app_root
 		if @dirs.empty?
-			return File.expand_path(".")
+			return absolute_path(".")
 		else
-			return File.expand_path(@dirs[0])
+			return absolute_path(@dirs[0])
 		end
 	end
-	
+
 	def load_config_file!(context, filename)
 		PhusionPassenger.require_passenger_lib 'utils/json' if !defined?(PhusionPassenger::Utils::JSON)
 		begin
@@ -168,11 +171,11 @@ private
 		STDERR.puts "*** Warning: #{e.message}"
 		return {}
 	end
-	
+
 	def looks_like_app_directory?(dir)
 		return AppFinder.looks_like_app_directory?(dir)
 	end
-	
+
 	def filename_to_server_names(filename)
 		basename = File.basename(filename)
 		names = [basename]
@@ -181,7 +184,7 @@ private
 		end
 		return names
 	end
-	
+
 	# Wait until the given IO becomes readable, or until the timeout has
 	# been reached. Returns true if the IO became readable, false if the
 	# timeout has been reached.
