@@ -36,9 +36,13 @@ module AnsiColors
 	BLACK_BG = "\e[40m".freeze
 	BLUE_BG  = "\e[44m".freeze
 	DEFAULT_TERMINAL_COLOR = "#{RESET}#{WHITE}#{BLACK_BG}".freeze
-	
+
 	extend self  # Make methods available as class methods.
-	
+
+	def self.new(type = :auto)
+		return AnsiColorsPrinter.new(type)
+	end
+
 	def self.included(klass)
 		# When included into another class, make sure that Utils
 		# methods are made private.
@@ -46,7 +50,7 @@ module AnsiColors
 			klass.send(:private, method_name)
 		end
 	end
-	
+
 	def ansi_colorize(text)
 		text = text.gsub(%r{<b>(.*?)</b>}m, "#{BOLD}\\1#{DEFAULT_TERMINAL_COLOR}")
 		text.gsub!(%r{<dgray>(.*?)</dgray>}m, "#{BOLD}#{DGRAY}\\1#{DEFAULT_TERMINAL_COLOR}")
@@ -70,5 +74,80 @@ module AnsiColors
 	end
 end
 
+class AnsiColorsPrinter
+	def initialize(enabled = :auto)
+		@enabled = enabled
+	end
+
+	def reset
+		return maybe_colorize(AnsiColors::RESET)
+	end
+
+	def bold
+		return maybe_colorize(AnsiColors::BOLD)
+	end
+
+	def dgray
+		return maybe_colorize(AnsiColors::DGRAY)
+	end
+
+	def red
+		return maybe_colorize(AnsiColors::RED)
+	end
+
+	def orange
+		return maybe_colorize(AnsiColors::ORANGE)
+	end
+
+	def green
+		return maybe_colorize(AnsiColors::GREEN)
+	end
+
+	def yellow
+		return maybe_colorize(AnsiColors::YELLOW)
+	end
+
+	def white
+		return maybe_colorize(AnsiColors::WHITE)
+	end
+
+	def black_bg
+		return maybe_colorize(AnsiColors::BLACK_BG)
+	end
+
+	def blue_bg
+		return maybe_colorize(AnsiColors::BLUE_BG)
+	end
+
+	def default_terminal_color
+		return maybe_colorize(AnsiColors::DEFAULT_TERMINAL_COLOR)
+	end
+
+	def ansi_colorize(text)
+		if should_output_color?
+			return AnsiColors.ansi_colorize(text)
+		else
+			return AnsiColors.strip_color_tags(text)
+		end
+	end
+
+private
+	def maybe_colorize(ansi_color)
+		if should_output_color?
+			return ansi_color
+		else
+			return ""
+		end
+	end
+
+	def should_output_color?
+		if @enabled == :auto
+			return STDOUT.tty?
+		else
+			return @enabled
+		end
+	end
+end
+
 end # module Utils
 end # module PhusionPassenger
