@@ -1,14 +1,15 @@
 require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')
 require 'ruby/shared/loader_sharedspec'
-require 'ruby/shared/rails/analytics_logging_extensions_sharedspec'
+require 'ruby/shared/rails/union_station_extensions_sharedspec'
 
+if RUBY_VERSION_INT >= 190
 module PhusionPassenger
 
-describe "Rack loader with Rails 3.2" do
+describe "Rack loader with Rails 4.0" do
 	include LoaderSpecHelper
 
 	before :each do
-		@stub = register_stub(RackStub.new("rails3.2"))
+		@stub = register_stub(RackStub.new("rails4.0"))
 	end
 
 	def start(options = {})
@@ -17,10 +18,11 @@ describe "Rack loader with Rails 3.2" do
 	end
 
 	def rails_version
-		return "3.2"
+		return "4.0"
 	end
 
-	include_examples "analytics logging extensions for Rails"
+	include_examples "Union Station extensions for Rails"
 end
 
 end # module PhusionPassenger
+end
